@@ -9,6 +9,7 @@ import warnings
 import urllib3
 import ssl
 from pprint import pprint
+import os
 
 import pyrena.classes as classes
 
@@ -131,8 +132,12 @@ class Arena():
         self.__dict__["_username"]=None
         self.__dict__["_password"]=None
         self.__dict__["me"]=None
+        self._workspace_id=None
+        self._workspace_name=NOne
+        self.headers["arena_session_id"]=None
+        self.reauth_utc=0
 
-    def _fetch(self, method, endpoint, params={}, data={}, files={}, headers={}, **kwargs):
+    def _fetch(self, method, endpoint, params={}, data={}, files={}, headers={}, expect_json=True, **kwargs):
 
         if not self._reqs_remaining:
             raise ArenaHTTPError("Request limit of 25000/day reached.")
@@ -179,6 +184,8 @@ class Arena():
 
         if method.lower()=="delete":
             return
+        elif expect_json==False:
+            return req.content
         else:
             return req.json()
 
@@ -245,6 +252,9 @@ class Arena():
 
         listing=[obj(**dictionary) for dictionary in data["results"]]
 
+        if not listing:
+            return []
+
         for item in listing:
             item._client=self
 
@@ -312,7 +322,7 @@ class Arena():
         results=self.Listing(obj_type, **kwargs)
 
         if len(results) != 1:
-            raise RuntimeError(f"{len(results)} instances of {obj_type.__class__.__name__} found")
+            raise RuntimeError(f"{len(results)} instances of {obj_type.__class__.__name__} found: {results}")
 
         return results[0]
 
@@ -351,6 +361,78 @@ class Arena():
             self.__dict__['me']=self.Listing(self.User, email=self.__dict__['_username'])[0]
             return self.__dict__['me']
 
+    def archive(self, folder_name="", exclude=[]):
+
+        """
+        WIP feature
+
+        Downloads all available data from the current Arena instance and saves it in JSON format.
+        Intended to be used as a rapid bulk export for local backup.
+
+        This may take some time, will verbosely output data, and may consume a large portion of the daily API limit.
+
+        Optional arguments:
+
+        - folder    - Path to existing archive folder, if one already exists
+        - exclude   - list of Arena classes to exclude from the backup.
+        """
+
+        d= time.strftime("%d_%B_%Y")
+
+        folder_name = folder_name or f"Arena_archive_{d}"
+
+        if not os.path.exists(folder_name):
+            os.mkdir(folder_name)
+
+        #Files
+        if self.File not in exclude:
+            files=self.Listing(self.File, limit=None)
+
+            with open(f"{folder_name}/files_{time.strftime('%d_%B_%Y')}.txt", "w+"):
+                file.write(json.dumps([x.json for x in files], indent=2))
+                file.truncate()
+
+            if not os.path.exists(f"{folder_name}/file_vault"):
+                os.mkdir(f"{folder_name}/file_vault")
+
+            for file in files:
+                with open(f"{folder_name}/file_vault/[{file.number}] {file.title}.{file.format}", "w+") as f:
+
+
+        #Item
+        if self.Item not in exclude:
+            items = self.Listing(self.Item, limit=None)
+            for item in items:
+                item.__dict__["files"]=[x.json for x in item.file_associations]
+                item.__dict__["revisions"]=[x.json for x in item.revisions]
+
+            with open(f"{folder_name}/items_{time.strftime('%d_%B_%Y')}.txt", "w+") as f:
+                f.write(json.dumps([x.json for x in items], indent=2))
+                f.truncate()
+
+        #Quality
+        if self.QualityProcess not in exclude:
+            qps = self.Listing(self.QualityProcess, limit=None)
+            for qp in qps:
+                for step in qp.steps:
+                    step.__dict__['affected']=[x.json for x in step.affected]
+                qp.__dict__["steps"]=[x.json for x in qp.steps]
+
+            with open(f"{folder_name}/quality_{time.strftime('%d_%B_%Y')}.txt", "w+") as f:
+                f.write(json.dumps([x.json for x in qps], indent=2))
+                f.truncate()
+
+        #Training
+        if self.TrainingPlan not in exclude:
+            tps = self.Listing(self.TrainingPlan, limit=None)
+            for tp in tps:
+                tp.__dict__["records"]=[x.json for x in tp.records]
+                tp.__dict__["items"]=[x.json for x in tp.item_associations]
+                tp.__dict__["users"]=[x.json for x in tp.user_associations]
+
+            with open(f"{folder_name}/training_{time.strftime('%d_%B_%Y')}.txt", "w+") as f:
+                f.write(json.dumps([x.json for x in tps], indent=2))
+                f.truncate()
 
 def docs():
 
